"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const src_1 = require("../src");
const shippingData = [
    [23, 19],
    {
        AS: [undefined, { IR: [[], {}], KP: [[], {}], SY: [[], {}] }],
        EU: [undefined, { BY: [[], {}], RU: [[], {}] }],
        NA: [[31, 29], { CU: [[], {}], US: [[41, 37], { CA: [[53, 51]] }] }],
        SA: [undefined, { VE: [[], {}] }],
    },
];
describe('Pricing Calculator', () => {
    it('should calculate the pricing of an example product with Customizations and shipping Correctly using main unit by default', () => {
        const exampleProduct = {
            price: 799,
            rewardShare: 19.5,
            startAmountPercent: 22.5,
            shipping: shippingData,
        };
        const exampleTraitCombination = { price: 1300 };
        const currency = { decimals: 2, symbol: 'usd' };
        const usedCreditReserve = 150;
        const usedCreditClaim = 150;
        const pricingInformation = (0, src_1.calculatePricingInformation)({
            items: [
                {
                    product: exampleProduct,
                    combination: exampleTraitCombination,
                    quantity: 1,
                },
            ],
            currency,
            usedCreditReserve,
            usedCreditClaim,
            stateCode: 'CA',
            countryCode: 'US',
        });
        expect(pricingInformation.items[0].reserve.base +
            pricingInformation.items[0].claim.base).toBe(pricingInformation.items[0].aggregate.base);
        expect(pricingInformation.summary.reserve.base +
            pricingInformation.summary.claim.base).toBe(pricingInformation.summary.aggregate.base);
        expect(pricingInformation.items[0].startAmountPercent).toBe(22.5);
        expect(pricingInformation.items[0].reserve.base).toBe(1.8);
        expect(pricingInformation.items[0].claim.base).toBe(6.19);
        expect(pricingInformation.items[0].aggregate.base).toBe(7.99);
        // reserve.base + traitCost + shippingCost + taxCost = reserve.total
        // 180 + 1300 + 5100 + 3815 = 10395
        expect(pricingInformation.items[0].reserve.total).toBe(103.95);
        // reserve.base + traitCost + shippingCost + taxCost - usedCreditReserve = summary.reserve.total
        // 180 + 1300 + 5100 + 3815 - 150 = 10245
        expect(pricingInformation.summary.reserve.total).toBe(102.45);
        expect(pricingInformation.items[0].reserve.taxCost).toBe(38.15);
        expect(pricingInformation.items[0].reserve.gainedCredit).toBe(0.35);
        expect(pricingInformation.items[0].claim.gainedCredit).toBe(1.21);
        expect(pricingInformation.items[0].aggregate.gainedCredit).toBe(1.56);
        // total.base + traitCost + shippingCost + taxCost = aggregate.total
        // 799 + 1300 + 5100 + 3815 = 11014
        expect(pricingInformation.items[0].aggregate.total).toBe(110.14);
        // total.base + traitCost + shippingCost + taxCost - usedCredit = summary.aggregate.total
        // 799 + 1300 + 5100 + 3815 - 300 = 10714
        expect(pricingInformation.summary.aggregate.total).toBe(107.14);
    });
    it('should calculate the pricing of an example product with customizations and shipping correctly with smallest unit if requested', () => {
        const exampleProduct = {
            price: 799,
            rewardShare: 19.5,
            startAmountPercent: 22.5,
            shipping: shippingData,
        };
        const exampleTraitCombination = { price: 1300 };
        const currency = { decimals: 2, symbol: 'usd' };
        const usedCreditReserve = 150;
        const usedCreditClaim = 150;
        const pricingInformation = (0, src_1.calculatePricingInformation)({
            items: [
                {
                    product: exampleProduct,
                    combination: exampleTraitCombination,
                    quantity: 1,
                },
            ],
            currency,
            usedCreditReserve,
            usedCreditClaim,
            stateCode: 'CA',
            countryCode: 'US',
        }, { returnSmallestUnit: true });
        expect(pricingInformation.items[0].reserve.base +
            pricingInformation.items[0].claim.base).toBe(pricingInformation.items[0].aggregate.base);
        expect(pricingInformation.items[0].startAmountPercent).toBe(22.5);
        expect(pricingInformation.items[0].reserve.base).toBe(180);
        expect(pricingInformation.items[0].claim.base).toBe(619);
        expect(pricingInformation.items[0].aggregate.base).toBe(799);
        // reserve.base + traitCost + shippingCost + taxCost = reserve.total
        // 180 + 1300 + 5100 + 3815 = 10395
        expect(pricingInformation.items[0].reserve.total).toBe(10395);
        // reserve.base + traitCost + shippingCost + taxCost - usedCredit = summary.reserve.total
        // 180 + 1300 + 5100 + 3815 - 150 = 10245
        expect(pricingInformation.summary.reserve.total).toBe(10245);
        expect(pricingInformation.items[0].reserve.taxCost).toBe(3815);
        expect(pricingInformation.items[0].reserve.gainedCredit).toBe(35);
        expect(pricingInformation.items[0].claim.gainedCredit).toBe(121);
        expect(pricingInformation.items[0].aggregate.gainedCredit).toBe(156);
        // total.base + traitCost + shippingCost + taxCost = aggregate.total
        // 799 + 1300 + 5100 + 3815 = 11014
        expect(pricingInformation.items[0].aggregate.total).toBe(11014);
        // total.base + traitCost + shippingCost + taxCost - usedCredit = summary.aggregate.total
        // 799 + 1300 + 5100 + 3815 - 300 = 10714
        expect(pricingInformation.summary.aggregate.total).toBe(10714);
    });
    it('should calculate the pricing of multiple products with customizations and shipping correctly using main unit by default', () => {
        const exampleProduct1 = {
            price: 799,
            rewardShare: 19.5,
            startAmountPercent: 22.5,
            shipping: shippingData,
        };
        const exampleTraitCombination1 = { price: 1300 };
        const exampleProduct2 = {
            price: 699,
            rewardShare: 11.5,
            startAmountPercent: 5.5,
            shipping: shippingData,
        };
        const exampleTraitCombination2 = { price: 1000 };
        const exampleProduct3 = {
            price: 850,
            rewardShare: 69.5,
            startAmountPercent: 3.5,
            shipping: shippingData,
        };
        const exampleTraitCombination3 = { price: 6666 };
        const currency = { decimals: 2, symbol: 'usd' };
        const usedCreditReserve = 222;
        const usedCreditClaim = 333;
        const pricingInformation = (0, src_1.calculatePricingInformation)({
            items: [
                {
                    product: exampleProduct1,
                    combination: exampleTraitCombination1,
                    quantity: 3,
                },
                {
                    product: exampleProduct2,
                    combination: exampleTraitCombination2,
                    quantity: 1,
                },
                {
                    product: exampleProduct3,
                    combination: exampleTraitCombination3,
                    quantity: 2,
                },
            ],
            currency,
            usedCreditReserve,
            usedCreditClaim,
            stateCode: 'CA',
            countryCode: 'US',
        });
        expect(pricingInformation.summary.reserve.base +
            pricingInformation.summary.claim.base).toBe(pricingInformation.summary.aggregate.base);
        expect(pricingInformation.items[0].startAmountPercent).toBe(22.5);
        expect(pricingInformation.items[0].reserve.base).toBe(5.39);
        expect(pricingInformation.items[0].claim.base).toBe(18.58);
        expect(pricingInformation.items[0].aggregate.base).toBe(23.97);
        // (reserve.base + traitCost + shippingCost + taxCost) * quantity = reserve.total
        // (180 + 1300 + 5100 + 3815) * 3 = 31184
        expect(pricingInformation.items[0].reserve.total).toBe(311.84);
        // [for all items: ((reserve.base + traitCost + shippingCost + taxCost) * quantity )] - usedCreditReserve = summary.reserve.total
        // ((180 + 1300 + 5100 + 3815) * 3 ) + (38 + 1000 + 5100 + 3603) + (60 + 13332 + 10200 + 13372) - 222 = 30962
        // 31184 + 9741 + 36964 - 222 = 77667
        expect(pricingInformation.summary.reserve.total).toBe(776.67);
        expect(pricingInformation.items[0].reserve.taxCost).toBe(114.45);
        expect(pricingInformation.items[0].reserve.gainedCredit).toBe(1.05);
        expect(pricingInformation.items[0].claim.gainedCredit).toBe(3.62);
        expect(pricingInformation.items[0].aggregate.gainedCredit).toBe(4.67);
        // (total.base + traitCost + shippingCost + taxCost) * quantity = aggregate.total
        // 2397 + 3900 + 15300 + 11445 = 33042
        expect(pricingInformation.items[0].aggregate.total).toBe(330.42);
        // [for all items: ((total.base + traitCost + shippingCost + taxCost) * 3)] - usedCredit = summary.aggregate.total
        // (2397 + 3900 + 15300 + 11445) + (699 + 1000 + 5100 + 3603) + (1700 + 13332 + 10200 + 13372) - 555 = 81493
        // 33042 + 10402 + 38604 - 555 = 81493
        expect(pricingInformation.summary.aggregate.total).toBe(814.93);
        // 3900 + 1000 + 13332 = 18232
        expect(pricingInformation.summary.reserve.traitCost).toBe(182.32);
        // 11445 + 3603 + 13372 = 28420
        expect(pricingInformation.summary.reserve.taxCost).toBe(284.2);
        // 15300 + 5100 + 10200 = 30600
        expect(pricingInformation.summary.reserve.shippingCost).toBe(306.0);
        // (156 * 3) + (81 * 1) + (590 * 2) = 1729
        expect(pricingInformation.summary.aggregate.gainedCredit).toBe(17.29);
        // ((156 * 3) * 0.225) + ((81 * 1) * 0.055) + ((590 * 2) * 0.035) = 151
        expect(pricingInformation.summary.reserve.gainedCredit).toBe(1.5);
        // ((156 * 3) * 0.775) + ((81 * 1) * 0.945) + ((590 * 2) * 0.965) = 1578
        expect(pricingInformation.summary.claim.gainedCredit).toBe(15.78);
    });
    it('should calculate credit of a product correctly', () => {
        const exampleProduct1 = {
            price: 799,
            rewardShare: 19.5,
            startAmountPercent: 22.5,
        };
        const currency = { decimals: 2, symbol: 'usd' };
        const credit = (0, src_1.calculateCredit)(exampleProduct1, 1, currency, {
            returnSmallestUnit: true,
        });
        expect(credit.gainedCreditReserve).toBe(35);
        expect(credit.gainedCreditClaim).toBe(121);
        expect(credit.gainedCredit).toBe(156);
        const exampleProduct2 = {
            price: 699,
            rewardShare: 11.5,
            startAmountPercent: 5.5,
            shipping: shippingData,
        };
        const credit2 = (0, src_1.calculateCredit)(exampleProduct2, 6, currency);
        expect(credit2.gainedCreditReserve).toBe(0.27);
        expect(credit2.gainedCreditClaim).toBe(4.56);
        expect(credit2.gainedCredit).toBe(4.82);
    });
});
